@@ -4,7 +4,9 @@ import math
 import matplotlib.pyplot as plt
 import warnings
 import utm
+from datetime import datetime
 from .utils import presence_3d
+from .feature_extraction import grouping_data, regrouping_data
 
 
 def from_dataframe(data, dictionary):
@@ -29,7 +31,9 @@ def interpolate(data,
                 limit=1,
                 limit_direction="forward",
                 inplace=False,
-                method="linear"):
+                method="linear",
+                order=1,
+                date_format=False):
     """
     Interpolate over missing values in pandas Dataframe of movement records.
     Interpolation methods consist of "linear", "polynomial, "time", "index", "pad".
@@ -38,15 +42,39 @@ def interpolate(data,
     :param data: Pandas DataFrame of movement records
     :param limit: Maximum number of consecutive NANs to fill
     :param limit_direction: If limit is specified, consecutive NaNs will be filled in this direction.
+    :param inplace: Update the data in place if possible.
     :param method: Interpolation technique to use. Default is "linear".
-    :param order: To be used in case of polynomial interpolation.
+    :param order: To be used in case of polynomial or spline interpolation.
+    :param date_format: Boolean to define whether time is some kind of date format. In this case column type has to be converted before calling interpolate.
     :return: Interpolated DataFrame.
     """
+    # converting time column if needed
+    if date_format:
+        if 'time' in data.columns:
+            timestamp_column = data['time'].apply(lambda x: x.timestamp())
+            time_difference = timestamp_column.apply(lambda x: datetime.fromtimestamp(x))[data['time'].first_valid_index()]\
+            - data['time'][data['time'].first_valid_index()]
+            data['time'] = data['time'].apply(lambda x: x.timestamp())
+        else:
+            warnings.warn('Please rename the time column to "time".')
+
+
     # Interpolating record data
-    interp = data.interpolate(limit=limit,
-                              limit_direction=limit_direction,
-                              inplace=inplace,
-                              method=method)
+    if method != "polynomial" and method != "spline":
+        interp = data.interpolate(limit=limit,
+                                  limit_direction=limit_direction,
+                                  inplace=inplace,
+                                  method=method)
+    else:
+        interp = data.interpolate(limit=limit,
+                                  limit_direction=limit_direction,
+                                  inplace=inplace,
+                                  method=method,
+                                  order=order)
+    # convert time column back to date
+    if date_format:
+        interp['time'] = interp['time'].apply(lambda x: datetime.fromtimestamp(x)) - time_difference
+
     return interp
 
 
@@ -56,7 +84,6 @@ def plot_missing_values(data):
     Plot the missing values of an animal-ID against time.
 
     :param data: Pandas DataFrame containing records of movement.
-    :param animal_id: ID of the animal whose missing values will be plotted against time.
     :return: None.
     """
     # Visualizing the count of missing values for all attributes-
@@ -74,19 +101,23 @@ def preprocess(data,
                limit=1,
                limit_direction="forward",
                inplace=False,
-               method="linear"):
+               method="linear",
+               order=1,
+               date_format = False):
     """
     Function to perform data preprocessing.
 
-    Print the number of missing values per column; Drop columns with  missing values for 'time' and 'animal_id';
+    Print the number of missing values per column; Drop columns with missing values for 'time' and 'animal_id';
     Remove the duplicated rows found.
     :param data: DataFrame to perform preprocessing on
     :param dropna: Optional parameter to drop columns with  missing values for 'time' and 'animal_id'
-    :param interpolate: Optional parameter to perform linear interpolation
+    :param interpolation: Optional parameter to perform interpolation
     :param limit: Maximum number of consecutive NANs to fill
     :param limit_direction: If limit is specified, consecutive NaNs will be filled in this direction.
+    :param inplace: Update the  data in place if possible.
     :param method: Interpolation technique to use. Default is "linear".
-    :param order: To be used in case of polynomial interpolation.
+    :param order: To be used in case of polynomial or spline interpolation.
+    :param date_format: Boolean to define whether time is some kind of date format. Important for interpolation.
     :return: Preprocessed DataFrame.
     """
     # Print the number of missing values per column
@@ -98,7 +129,9 @@ def preprocess(data,
                            limit=limit,
                            limit_direction=limit_direction,
                            inplace=inplace,
-                           method=method)
+                           method=method,
+                           order=order,
+                           date_format=date_format)
 
     # Drop columns with  missing values for 'time'  and 'animal_id'
 
@@ -133,7 +166,7 @@ def print_duplicate(df):
     """
     dup = df[df.duplicated(['time', 'animal_id'])]
     print(
-        "Removed duplicate rows based on the columns 'animal_id' and 'time' column are:",
+        "Duplicate rows based on the columns 'animal_id' and 'time' column are:",
         dup,
         sep='\n')
 
@@ -143,7 +176,7 @@ def filter_dataframe(data, frm, to):
     Extract records of assigned time frame from preprocessed movement record data.
 
     :param data: Pandas DataFrame, containing preprocessed movement record data.
-    :param frm: Int, defining starting point from where to extract records.
+    :param frm: Int, defining starting point from where to extract records.Note that if time is stored as a date (if input data has time not stored as numeric type it is automatically converted to datetime) parameter has to be set using an datetime format: mkit.filter_dataframe(data, "2008-01-01", "2010-10-01")
     :param to: Int, defining end point up to where to extract records.
     :return: Pandas DataFrame, filtered by records matching the defined frame in 'from'-'to'.
     """
@@ -161,19 +194,23 @@ def replace_parts_animal_movement(data_groups, animal_id, time_array,
         arr_index = np.array([10, 20, 200, 20000, 40000, 43200])
         replaced_data_groups = replace_parts_animal_movement(data_groups, 811, arr_index, 100, 90)
 
-    :param data_groups: Dictionary with key 'animal_id'and value with records for 'animal_id'.
+    :param data_groups: DataFrame containing the movement records.
     :param animal_id: Int defining 'animal_id' whose movements have to be replaced.
     :param time_array: Array defining time indices whose movements have to replaced
     :param replacement_value_x: Int value that will replace all 'x' attribute values in 'time_array'.
     :param replacement_value_y: Int value that will replace all 'y' attribute values in 'time_array'.
+    :param replacement_value_z: Int value that will replace all 'z' attribute values in 'time_array'. (optional)
     :return: Dictionary with replaced subsets.
     """
-    data_groups[animal_id].loc[time_array, 'x'] = replacement_value_x
-    data_groups[animal_id].loc[time_array, 'y'] = replacement_value_y
-    if presence_3d(data_groups[animal_id]):
-        data_groups[animal_id].loc[time_array, 'z'] = replacement_value_z
+    # Grouping DataFrame
+    data_groups = grouping_data(data_groups, preprocessedMethod=True)
 
-    return data_groups
+    data_groups[animal_id].loc[data_groups[animal_id]["time"].isin(time_array), 'x'] = replacement_value_x
+    data_groups[animal_id].loc[data_groups[animal_id]["time"].isin(time_array), 'y'] = replacement_value_y
+    if presence_3d(data_groups[animal_id]):
+        data_groups[animal_id].loc[data_groups[animal_id]["time"].isin(time_array), 'z'] = replacement_value_z
+
+    return regrouping_data(data_groups)
 
 
 def resample_systematic(data_groups, downsample_size):
@@ -183,10 +220,13 @@ def resample_systematic(data_groups, downsample_size):
     This is done to reduce the resolution of the dataset. This function does this by systematically choosing
     samples from each animal.
 
-    :param data_groups: Dictionary with key: 'animal_id' and value with record data to that 'animal_id'.
+    :param data_groups: DataFrame containing the movement records.
     :param downsample_size: Int sample size to which each animal has to be reduced by downsampling.
-    :return: Dictionary, modified from original size 'data_groups' to 'downsample_size'.
+    :return: DataFrame, modified from original size 'data_groups' to 'downsample_size'.
     """
+    # group the dataFrame
+    data_groups = grouping_data(data_groups, preprocessedMethod=True)
+
     # Get first key-
     first = list(data_groups.keys())[0]
 
@@ -210,6 +250,8 @@ def resample_systematic(data_groups, downsample_size):
     for aid in data_groups.keys():
         data_groups_downsampled[aid] = data_groups[aid].loc[arr_index, :]
 
+    data_groups_downsampled = regrouping_data(data_groups_downsampled)
+
     return data_groups_downsampled
 
 
@@ -220,10 +262,13 @@ def resample_random(data_groups, downsample_size):
     This is done to reduce resolution of the dataset. This function does this by randomly choosing
     samples from each animal
 
-    :param data_groups: Dictionary with key 'animal_id' and value record data of 'animal_id'.
+    :param data_groups: DataFrame containing the movement records.
     :param downsample_size: Int sample size to which each animal has to be reduced by downsampling.
-    :return: Dictionary, modified from original size 'data_groups' to 'downsample_size'.
+    :return: DataFrame, modified from original size 'data_groups' to 'downsample_size'.
     """
+    # group the dataFrame
+    data_groups = grouping_data(data_groups, preprocessedMethod=True)
+
     # Get first key-
     first = list(data_groups.keys())[0]
 
@@ -239,6 +284,8 @@ def resample_random(data_groups, downsample_size):
     for aid in data_groups.keys():
         data_groups_downsampled[aid] = data_groups[aid].loc[ix_random, :]
 
+    data_groups_downsampled = regrouping_data(data_groups_downsampled)
+
     return data_groups_downsampled
 
 
@@ -250,12 +297,15 @@ def split_trajectories(data_groups, segment, fuzzy_segment=0, csv=False):
         data_groups = group_animals(data)
         split_trajectories_fuzzy_segmentation(data_groups, segment = 5, fuzzy_segment = 5)
 
-    :param data_groups: Dictionary with key 'animal_id' and value record data of 'animal_id'.
+    :param data_groups: DataFrame with movement records.
     :param segment: Int, defining point where the animals are split into several Pandas Data Frames.
     :param fuzzy_segment: Int, defining interval which will overlap on either side of the segments.
     :param csv: Boolean, defining if each interval shall be exported locally as singular csv
-    :return: None. All segmented Pandas Data Frames are saved to HDD.
+    :return: Dictionary with the created DataFrames for each animal.
     """
+    # Group the DataFrame
+    data_groups = grouping_data(data_groups, preprocessedMethod=True)
+
     # Get first key-
     first = list(data_groups.keys())[0]
 
@@ -280,7 +330,7 @@ def split_trajectories(data_groups, segment, fuzzy_segment=0, csv=False):
     return groups
 
 
-def convert_measueres(preprocessed_data, x_min=0, x_max=1, y_min=0, y_max=1):
+def convert_measueres(preprocessed_data, x_min=0, x_max=1, y_min=0, y_max=1, z_min=0, z_max=1):
     """
     Create a linear scale with input parameters for x,y for transformation of position data.
     :param preprocessed_data: Pandas DataFrame only with x and y position data
@@ -288,13 +338,15 @@ def convert_measueres(preprocessed_data, x_min=0, x_max=1, y_min=0, y_max=1):
     :param x_max: int maximum for x - default: 1.
     :param y_min: int minimum for y - default: 0.
     :param y_max: int maximum for y - default: 1.
+    :param z_min: int minimum for z - default: 0.
+    :param z_max: int maximum for z - default: 1.
     :return: Pandas DataFrame with linearly transformed position data.
     """
     # Preventing features input along position data
-    if [*preprocessed_data.columns] != ['time', 'animal_id', 'x', 'y']:
+    if [*preprocessed_data.columns] != ['time', 'animal_id', 'x', 'y'] and [*preprocessed_data.columns] != ['time', 'animal_id', 'x', 'y', 'z']:
         print(
             "\nError! Conversion only allowed for dataframes with colnames ['time', 'animal_id', 'x', "
-            "'y']. \n")
+            "'y'] or ['time', 'animal_id', 'x', 'y', 'z']. \n")
         return None
 
     # Linear Transformation of position dimensions
@@ -306,6 +358,12 @@ def convert_measueres(preprocessed_data, x_min=0, x_max=1, y_min=0, y_max=1):
         preprocessed_data.loc[:, "y"], (preprocessed_data.loc[:, "y"].min(),
                                         preprocessed_data.loc[:, "y"].max()),
         (y_min, y_max))
+
+    if presence_3d(preprocessed_data):
+        preprocessed_data.loc[:, "z"] = np.interp(
+            preprocessed_data.loc[:, "z"], (preprocessed_data.loc[:, "z"].min(),
+                                            preprocessed_data.loc[:, "z"].max()),
+            (z_min, z_max))
     return preprocessed_data
 
 
@@ -367,4 +425,4 @@ def convert_latlon(data, latitude='latitude', longitude='longitude', replace=Tru
         data.drop(latitude, axis=1, inplace=True)
         data.drop(longitude, axis=1, inplace=True)
 
-    return data
\ No newline at end of file
+    return data
